@@ -1,7 +1,7 @@
 use async_trait::async_trait;
-use chrono::prelude::*;
 use serde_json::Value;
 use std::collections::BTreeMap;
+use std::str::FromStr;
 
 use ssi::caip10::BlockchainAccountId;
 use ssi::caip2::ChainId;
@@ -14,15 +14,22 @@ use ssi::did_resolve::{
 };
 use ssi::jwk::{Base64urlUInt, OctetParams, Params, JWK};
 
+// https://github.com/ChainAgnostic/CAIPs/blob/master/CAIPs/caip-3.md
+const REFERENCE_EIP155_ETHEREUM_MAINNET: &str = "1";
+
+const REFERENCE_EIP155_CELO_MAINNET: &str = "42220";
+const REFERENCE_EIP155_POLYGON_MAINNET: &str = "137";
+
 // https://github.com/ChainAgnostic/CAIPs/blob/master/CAIPs/caip-4.md
-const MAINNET_BITCOIN: &str = "000000000019d6689c085ae165831e93";
-const MAINNET_DOGECOIN: &str = "1a91e3dace36e2be3bf030a65679fe82";
+const REFERENCE_BIP122_BITCOIN_MAINNET: &str = "000000000019d6689c085ae165831e93";
+
+const REFERENCE_BIP122_DOGECOIN_MAINNET: &str = "1a91e3dace36e2be3bf030a65679fe82";
 
 // https://github.com/ChainAgnostic/CAIPs/blob/master/CAIPs/caip-26.md
-const MAINNET_TEZOS: &str = "NetXdQprcVkpaWU";
+const REFERENCE_TEZOS_MAINNET: &str = "NetXdQprcVkpaWU";
 
 // https://github.com/ChainAgnostic/CAIPs/blob/master/CAIPs/caip-30.md
-const MAINNET_SOLANA: &str = "4sGjMW1sUnHzSxGspuhpqLDx6wiyjNtZ";
+const REFERENCE_SOLANA_MAINNET: &str = "4sGjMW1sUnHzSxGspuhpqLDx6wiyjNtZ";
 
 /// did:pkh DID Method
 pub struct DIDPKH;
@@ -47,21 +54,21 @@ fn resolution_error(err: &str) -> ResolutionResult {
     (ResolutionMetadata::from_error(err), None, None)
 }
 
-async fn resolve_tz(did: &str, account_address: String) -> ResolutionResult {
+async fn resolve_tezos(did: &str, account_address: String, reference: &str) -> ResolutionResult {
     if account_address.len() < 3 {
-        return resolution_error(&ERROR_INVALID_DID);
+        return resolution_error(ERROR_INVALID_DID);
     }
-    let (vm_type, vm_type_iri) = match &account_address[0..3] {
-        "tz1" => ("Ed25519PublicKeyBLAKE2BDigestSize20Base58CheckEncoded2021", "https://w3id.org/security#Ed25519PublicKeyBLAKE2BDigestSize20Base58CheckEncoded2021"),
-        "tz2" => ("EcdsaSecp256k1RecoveryMethod2020", "https://identity.foundation/EcdsaSecp256k1RecoverySignature2020#EcdsaSecp256k1RecoveryMethod2020"),
-        "tz3" => ("P256PublicKeyBLAKE2BDigestSize20Base58CheckEncoded2021", "https://w3id.org/security#P256PublicKeyBLAKE2BDigestSize20Base58CheckEncoded2021"),
-        _ => return resolution_error(&ERROR_INVALID_DID),
+    let (vm_type, vm_type_iri) = match account_address.get(0..3) {
+        Some("tz1") => ("Ed25519PublicKeyBLAKE2BDigestSize20Base58CheckEncoded2021", "https://w3id.org/security#Ed25519PublicKeyBLAKE2BDigestSize20Base58CheckEncoded2021"),
+        Some("tz2") => ("EcdsaSecp256k1RecoveryMethod2020", "https://identity.foundation/EcdsaSecp256k1RecoverySignature2020#EcdsaSecp256k1RecoveryMethod2020"),
+        Some("tz3") => ("P256PublicKeyBLAKE2BDigestSize20Base58CheckEncoded2021", "https://w3id.org/security#P256PublicKeyBLAKE2BDigestSize20Base58CheckEncoded2021"),
+        _ => return resolution_error(ERROR_INVALID_DID),
     };
     let blockchain_account_id = BlockchainAccountId {
         account_address,
         chain_id: ChainId {
             namespace: "tezos".to_string(),
-            reference: MAINNET_TEZOS.to_string(),
+            reference: reference.to_string(),
         },
     };
     let mut context = BTreeMap::new();
@@ -121,9 +128,14 @@ async fn resolve_tz(did: &str, account_address: String) -> ResolutionResult {
     resolution_result(doc)
 }
 
-async fn resolve_eth(did: &str, account_address: String) -> ResolutionResult {
+async fn resolve_eip155(
+    did: &str,
+    account_address: String,
+    reference: &str,
+    legacy: bool,
+) -> ResolutionResult {
     if !account_address.starts_with("0x") {
-        return resolution_error(&ERROR_INVALID_DID);
+        return resolution_error(ERROR_INVALID_DID);
     }
     let mut context = BTreeMap::new();
     context.insert(
@@ -138,12 +150,19 @@ async fn resolve_eth(did: &str, account_address: String) -> ResolutionResult {
         account_address,
         chain_id: ChainId {
             namespace: "eip155".to_string(),
-            reference: "1".to_string(),
+            reference: reference.to_string(),
         },
     };
+    let vm_fragment = if legacy {
+        // Explanation of fragment differences:
+        //   https://github.com/spruceid/ssi/issues/297
+        "Recovery2020"
+    } else {
+        "blockchainAccountId"
+    };
     let vm_url = DIDURL {
         did: did.to_string(),
-        fragment: Some("Recovery2020".to_string()),
+        fragment: Some(vm_fragment.to_string()),
         ..Default::default()
     };
     let vm = VerificationMethod::Map(VerificationMethodMap {
@@ -191,106 +210,18 @@ async fn resolve_eth(did: &str, account_address: String) -> ResolutionResult {
     resolution_result(doc)
 }
 
-async fn resolve_celo(did: &str, account_address: String) -> ResolutionResult {
-    if !account_address.starts_with("0x") {
-        return resolution_error(&ERROR_INVALID_DID);
-    }
-    let mut context = BTreeMap::new();
-    context.insert(
-        "blockchainAccountId".to_string(),
-        Value::String("https://w3id.org/security#blockchainAccountId".to_string()),
-    );
-    context.insert(
-        "EcdsaSecp256k1RecoveryMethod2020".to_string(),
-        Value::String("https://identity.foundation/EcdsaSecp256k1RecoverySignature2020#EcdsaSecp256k1RecoveryMethod2020".to_string()),
-    );
-    let blockchain_account_id = BlockchainAccountId {
-        account_address,
-        chain_id: ChainId {
-            namespace: "eip155".to_string(),
-            reference: "42220".to_string(),
-        },
-    };
-    let vm_url = DIDURL {
-        did: did.to_string(),
-        fragment: Some("Recovery2020".to_string()),
-        ..Default::default()
-    };
-    let vm = VerificationMethod::Map(VerificationMethodMap {
-        id: vm_url.to_string(),
-        type_: "EcdsaSecp256k1RecoveryMethod2020".to_string(),
-        controller: did.to_string(),
-        blockchain_account_id: Some(blockchain_account_id.to_string()),
-        ..Default::default()
-    });
-    let doc = Document {
-        context: Contexts::Many(vec![
-            Context::URI(DEFAULT_CONTEXT.to_string()),
-            Context::Object(context),
-        ]),
-        id: did.to_string(),
-        verification_method: Some(vec![vm]),
-        authentication: Some(vec![VerificationMethod::DIDURL(vm_url.clone())]),
-        assertion_method: Some(vec![VerificationMethod::DIDURL(vm_url)]),
-        ..Default::default()
-    };
-    resolution_result(doc)
-}
-
-async fn resolve_poly(did: &str, account_address: String) -> ResolutionResult {
-    if !account_address.starts_with("0x") {
-        return resolution_error(&ERROR_INVALID_DID);
-    }
-    let mut context = BTreeMap::new();
-    context.insert(
-        "blockchainAccountId".to_string(),
-        Value::String("https://w3id.org/security#blockchainAccountId".to_string()),
-    );
-    context.insert(
-        "EcdsaSecp256k1RecoveryMethod2020".to_string(),
-        Value::String("https://identity.foundation/EcdsaSecp256k1RecoverySignature2020#EcdsaSecp256k1RecoveryMethod2020".to_string()),
-    );
-    let blockchain_account_id = BlockchainAccountId {
-        account_address,
-        chain_id: ChainId {
-            namespace: "eip155".to_string(),
-            reference: "137".to_string(),
-        },
-    };
-    let vm_url = DIDURL {
-        did: did.to_string(),
-        fragment: Some("Recovery2020".to_string()),
-        ..Default::default()
-    };
-    let vm = VerificationMethod::Map(VerificationMethodMap {
-        id: vm_url.to_string(),
-        type_: "EcdsaSecp256k1RecoveryMethod2020".to_string(),
-        controller: did.to_string(),
-        blockchain_account_id: Some(blockchain_account_id.to_string()),
-        ..Default::default()
-    });
-    let doc = Document {
-        context: Contexts::Many(vec![
-            Context::URI(DEFAULT_CONTEXT.to_string()),
-            Context::Object(context),
-        ]),
-        id: did.to_string(),
-        verification_method: Some(vec![vm]),
-        authentication: Some(vec![VerificationMethod::DIDURL(vm_url.clone())]),
-        assertion_method: Some(vec![VerificationMethod::DIDURL(vm_url)]),
-        ..Default::default()
-    };
-    resolution_result(doc)
-}
-
-async fn resolve_sol(did: &str, account_address: String) -> ResolutionResult {
+async fn resolve_solana(did: &str, account_address: String, reference: &str) -> ResolutionResult {
     let public_key_bytes = match bs58::decode(&account_address).into_vec() {
         Ok(bytes) => bytes,
-        Err(_) => return resolution_error(&ERROR_INVALID_DID),
+        Err(_) => return resolution_error(ERROR_INVALID_DID),
     };
     if public_key_bytes.len() != 32 {
-        return resolution_error(&ERROR_INVALID_DID);
+        return resolution_error(ERROR_INVALID_DID);
     }
+    let chain_id = ChainId {
+        namespace: "solana".to_string(),
+        reference: reference.to_string(),
+    };
     let mut context = BTreeMap::new();
     context.insert(
         "blockchainAccountId".to_string(),
@@ -328,10 +259,7 @@ async fn resolve_sol(did: &str, account_address: String) -> ResolutionResult {
     };
     let blockchain_account_id = BlockchainAccountId {
         account_address,
-        chain_id: ChainId {
-            namespace: "solana".to_string(),
-            reference: MAINNET_SOLANA.to_string(),
-        },
+        chain_id,
     };
     let vm_url = DIDURL {
         did: did.to_string(),
@@ -354,7 +282,7 @@ async fn resolve_sol(did: &str, account_address: String) -> ResolutionResult {
     let solvm = VerificationMethod::Map(VerificationMethodMap {
         id: solvm_url.to_string(),
         type_: "SolanaMethod2021".to_string(),
-        public_key_jwk: Some(pk_jwk.clone()),
+        public_key_jwk: Some(pk_jwk),
         controller: did.to_string(),
         blockchain_account_id: Some(blockchain_account_id.to_string()),
         ..Default::default()
@@ -379,15 +307,27 @@ async fn resolve_sol(did: &str, account_address: String) -> ResolutionResult {
     resolution_result(doc)
 }
 
-async fn resolve_btc(did: &str, account_address: String) -> ResolutionResult {
-    if !account_address.starts_with("1") {
-        return resolution_error(&ERROR_INVALID_DID);
-    };
+async fn resolve_bip122(did: &str, account_address: String, reference: &str) -> ResolutionResult {
+    match reference {
+        REFERENCE_BIP122_BITCOIN_MAINNET => {
+            if !account_address.starts_with('1') {
+                return resolution_error(ERROR_INVALID_DID);
+            }
+        }
+        REFERENCE_BIP122_DOGECOIN_MAINNET => {
+            if !account_address.starts_with('D') {
+                return resolution_error(ERROR_INVALID_DID);
+            }
+        }
+        _ => {
+            // Unknown network address: no prefix hash check
+        }
+    }
     let blockchain_account_id = BlockchainAccountId {
         account_address,
         chain_id: ChainId {
             namespace: "bip122".to_string(),
-            reference: MAINNET_BITCOIN.to_string(),
+            reference: reference.to_string(),
         },
     };
     let vm_url = DIDURL {
@@ -425,25 +365,34 @@ async fn resolve_btc(did: &str, account_address: String) -> ResolutionResult {
     resolution_result(doc)
 }
 
-async fn resolve_doge(did: &str, account_address: String) -> ResolutionResult {
-    if !account_address.starts_with("D") {
-        return resolution_error(&ERROR_INVALID_DID);
+async fn resolve_aleo(did: &str, account_address: String, reference: &str) -> ResolutionResult {
+    use bech32::FromBase32;
+    let (hrp, data, _variant) = match bech32::decode(&account_address) {
+        Err(_e) => return resolution_error(ERROR_INVALID_DID),
+        Ok(data) => data,
+    };
+    if data.is_empty() {
+        return resolution_error(ERROR_INVALID_DID);
     }
-    let mut context = BTreeMap::new();
-    context.insert(
-        "blockchainAccountId".to_string(),
-        Value::String("https://w3id.org/security#blockchainAccountId".to_string()),
-    );
-    context.insert(
-        "EcdsaSecp256k1RecoveryMethod2020".to_string(),
-        Value::String("https://identity.foundation/EcdsaSecp256k1RecoverySignature2020#EcdsaSecp256k1RecoveryMethod2020".to_string()),
-    );
+    if hrp != "aleo" {
+        return resolution_error(ERROR_INVALID_DID);
+    }
+    let data = match Vec::<u8>::from_base32(&data) {
+        Err(_e) => return resolution_error(ERROR_INVALID_DID),
+        Ok(data) => data,
+    };
+    // Address data is decoded for validation only.
+    // The verification method object just uses the account address in blockchainAccountId.
+    if data.len() != 32 {
+        return resolution_error(ERROR_INVALID_DID);
+    }
+    let chain_id = ChainId {
+        namespace: "aleo".to_string(),
+        reference: reference.to_string(),
+    };
     let blockchain_account_id = BlockchainAccountId {
         account_address,
-        chain_id: ChainId {
-            namespace: "bip122".to_string(),
-            reference: MAINNET_DOGECOIN.to_string(),
-        },
+        chain_id,
     };
     let vm_url = DIDURL {
         did: did.to_string(),
@@ -451,8 +400,8 @@ async fn resolve_doge(did: &str, account_address: String) -> ResolutionResult {
         ..Default::default()
     };
     let vm = VerificationMethod::Map(VerificationMethodMap {
-        id: String::from(vm_url.clone()),
-        type_: "EcdsaSecp256k1RecoveryMethod2020".to_string(),
+        id: vm_url.to_string(),
+        type_: "BlockchainVerificationMethod2021".to_string(),
         controller: did.to_string(),
         blockchain_account_id: Some(blockchain_account_id.to_string()),
         ..Default::default()
@@ -460,7 +409,7 @@ async fn resolve_doge(did: &str, account_address: String) -> ResolutionResult {
     let doc = Document {
         context: Contexts::Many(vec![
             Context::URI(DEFAULT_CONTEXT.to_string()),
-            Context::Object(context),
+            Context::URI("https://w3id.org/security/suites/blockchain-2021/v1".to_string()),
         ]),
         id: did.to_string(),
         verification_method: Some(vec![vm]),
@@ -471,6 +420,23 @@ async fn resolve_doge(did: &str, account_address: String) -> ResolutionResult {
     resolution_result(doc)
 }
 
+async fn resolve_caip10(did: &str, account_id: String) -> ResolutionResult {
+    let account_id = match BlockchainAccountId::from_str(&account_id) {
+        Ok(account_id) => account_id,
+        Err(_) => return resolution_error(ERROR_INVALID_DID),
+    };
+    let namespace = account_id.chain_id.namespace;
+    let reference = account_id.chain_id.reference;
+    match &namespace[..] {
+        "tezos" => resolve_tezos(did, account_id.account_address, &reference).await,
+        "eip155" => resolve_eip155(did, account_id.account_address, &reference, false).await,
+        "bip122" => resolve_bip122(did, account_id.account_address, &reference).await,
+        "solana" => resolve_solana(did, account_id.account_address, &reference).await,
+        "aleo" => resolve_aleo(did, account_id.account_address, &reference).await,
+        _ => resolution_error(ERROR_INVALID_DID),
+    }
+}
+
 #[cfg_attr(target_arch = "wasm32", async_trait(?Send))]
 #[cfg_attr(not(target_arch = "wasm32"), async_trait)]
 impl DIDResolver for DIDPKH {
@@ -481,18 +447,20 @@ impl DIDResolver for DIDPKH {
     ) -> ResolutionResult {
         let (type_, data) = match did.splitn(4, ':').collect::<Vec<&str>>().as_slice() {
             ["did", "pkh", type_, data] => (type_.to_string(), data.to_string()),
-            _ => return resolution_error(&ERROR_INVALID_DID),
+            _ => return resolution_error(ERROR_INVALID_DID),
         };
 
         match &type_[..] {
-            "tz" => resolve_tz(did, data).await,
-            "eth" => resolve_eth(did, data).await,
-            "celo" => resolve_celo(did, data).await,
-            "poly" => resolve_poly(did, data).await,
-            "sol" => resolve_sol(did, data).await,
-            "btc" => resolve_btc(did, data).await,
-            "doge" => resolve_doge(did, data).await,
-            _ => resolution_error(&ERROR_INVALID_DID),
+            // Non-CAIP-10 (deprecated)
+            "tz" => resolve_tezos(did, data, REFERENCE_TEZOS_MAINNET).await,
+            "eth" => resolve_eip155(did, data, REFERENCE_EIP155_ETHEREUM_MAINNET, true).await,
+            "celo" => resolve_eip155(did, data, REFERENCE_EIP155_CELO_MAINNET, true).await,
+            "poly" => resolve_eip155(did, data, REFERENCE_EIP155_POLYGON_MAINNET, true).await,
+            "sol" => resolve_solana(did, data, REFERENCE_SOLANA_MAINNET).await,
+            "btc" => resolve_bip122(did, data, REFERENCE_BIP122_BITCOIN_MAINNET).await,
+            "doge" => resolve_bip122(did, data, REFERENCE_BIP122_DOGECOIN_MAINNET).await,
+            // CAIP-10
+            _ => resolve_caip10(did, type_ + ":" + &data).await,
         }
     }
 
@@ -513,7 +481,7 @@ fn generate_sol(jwk: &JWK) -> Option<String> {
 fn generate_btc(key: &JWK) -> Result<String, String> {
     let addr = ssi::ripemd::hash_public_key(key, 0x00)?;
     #[cfg(test)]
-    if !addr.starts_with("1") {
+    if !addr.starts_with('1') {
         return Err("Expected Bitcoin address".to_string());
     }
     Ok(addr)
@@ -522,15 +490,144 @@ fn generate_btc(key: &JWK) -> Result<String, String> {
 fn generate_doge(key: &JWK) -> Result<String, String> {
     let addr = ssi::ripemd::hash_public_key(key, 0x1e)?;
     #[cfg(test)]
-    if !addr.starts_with("D") {
+    if !addr.starts_with('D') {
         return Err("Expected Dogecoin address".to_string());
     }
     Ok(addr)
 }
 
+fn generate_caip10_tezos(
+    key: &JWK,
+    ref_opt: Option<String>,
+) -> Result<BlockchainAccountId, String> {
+    let hash = ssi::blakesig::hash_public_key(key)?;
+    let reference = ref_opt.unwrap_or_else(|| REFERENCE_TEZOS_MAINNET.to_string());
+    Ok(BlockchainAccountId {
+        account_address: hash,
+        chain_id: ChainId {
+            namespace: "tezos".to_string(),
+            reference,
+        },
+    })
+}
+
+fn generate_caip10_eip155(
+    key: &JWK,
+    ref_opt: Option<String>,
+) -> Result<BlockchainAccountId, String> {
+    let hash = ssi::keccak_hash::hash_public_key_eip55(key)?;
+    let reference = ref_opt.unwrap_or_else(|| REFERENCE_EIP155_ETHEREUM_MAINNET.to_string());
+    Ok(BlockchainAccountId {
+        account_address: hash,
+        chain_id: ChainId {
+            namespace: "eip155".to_string(),
+            reference,
+        },
+    })
+}
+
+fn generate_caip10_bip122(
+    key: &JWK,
+    ref_opt: Option<String>,
+) -> Result<BlockchainAccountId, String> {
+    let reference = ref_opt.unwrap_or_else(|| REFERENCE_BIP122_BITCOIN_MAINNET.to_string());
+    let addr;
+    match &reference[..] {
+        REFERENCE_BIP122_BITCOIN_MAINNET => {
+            addr = ssi::ripemd::hash_public_key(key, 0x00)?;
+            if !addr.starts_with('1') {
+                return Err("Expected Bitcoin address".to_string());
+            }
+        }
+        REFERENCE_BIP122_DOGECOIN_MAINNET => {
+            addr = ssi::ripemd::hash_public_key(key, 0x1e)?;
+            if !addr.starts_with('D') {
+                return Err("Expected Dogecoin address".to_string());
+            }
+        }
+        _ => {
+            return Err("Expected Bitcoin address type".to_string());
+        }
+    }
+
+    Ok(BlockchainAccountId {
+        account_address: addr,
+        chain_id: ChainId {
+            namespace: "bip122".to_string(),
+            reference,
+        },
+    })
+}
+
+fn generate_caip10_solana(
+    key: &JWK,
+    ref_opt: Option<String>,
+) -> Result<BlockchainAccountId, String> {
+    let reference = ref_opt.unwrap_or_else(|| "".to_string());
+    let chain_id = ChainId {
+        namespace: "solana".to_string(),
+        reference,
+    };
+    let pk_bs58 = match key.params {
+        Params::OKP(ref params) if params.curve == "Ed25519" => {
+            bs58::encode(&params.public_key.0).into_string()
+        }
+        _ => return Err("Invalid public key type for Solana".to_string()),
+    };
+    Ok(BlockchainAccountId {
+        account_address: pk_bs58,
+        chain_id,
+    })
+}
+
+fn generate_caip10_aleo(key: &JWK, ref_opt: Option<String>) -> Result<BlockchainAccountId, String> {
+    let reference = ref_opt.unwrap_or_else(|| "1".to_string());
+    let chain_id = ChainId {
+        namespace: "aleo".to_string(),
+        reference,
+    };
+    use bech32::ToBase32;
+    let pk_bs58 = match key.params {
+        Params::OKP(ref params) if params.curve == "AleoTestnet1Key" => bech32::encode(
+            "aleo",
+            &params.public_key.0.to_base32(),
+            bech32::Variant::Bech32m,
+        )
+        .unwrap(),
+        _ => return Err("Invalid public key type for Aleo".to_string()),
+    };
+    Ok(BlockchainAccountId {
+        account_address: pk_bs58,
+        chain_id,
+    })
+}
+
+fn generate_caip10_did(key: &JWK, name: &str) -> Result<String, String> {
+    // Require name to be a either CAIP-2 namespace or a
+    // full CAIP-2 string - namespace and reference (e.g. internal
+    // chain id or genesis hash).
+    // If reference is not provided, default to a known mainnet.
+    // If a reference is provided, pass it through.
+    // Return a CAIP-10 string, appended to "did:pkh:".
+    let (namespace, reference_opt) = match name.splitn(2, ':').collect::<Vec<&str>>().as_slice() {
+        [namespace] => (namespace.to_string(), None),
+        [namespace, reference] => (namespace.to_string(), Some(reference.to_string())),
+        _ => return Err("Unable to parse chain id or namespace".to_string()),
+    };
+    let account_id = match &namespace[..] {
+        "tezos" => generate_caip10_tezos(key, reference_opt)?,
+        "eip155" => generate_caip10_eip155(key, reference_opt)?,
+        "bip122" => generate_caip10_bip122(key, reference_opt)?,
+        "solana" => generate_caip10_solana(key, reference_opt)?,
+        "aleo" => generate_caip10_aleo(key, reference_opt)?,
+        _ => return Err("Namespace not supported".to_string()),
+    };
+    Ok(format!("did:pkh:{}", account_id))
+}
+
 impl DIDMethod for DIDPKH {
     fn name(&self) -> &'static str {
-        return "pkh";
+        "pkh"
     }
 
     fn generate(&self, source: &Source) -> Option<String> {
@@ -539,6 +636,7 @@ impl DIDMethod for DIDPKH {
             _ => return None,
         };
         let addr = match match &pkh_name[..] {
+            // Aliases for did:pkh pre-CAIP-10. Deprecate?
             "tz" => ssi::blakesig::hash_public_key(key).ok(),
             "eth" => ssi::keccak_hash::hash_public_key(key).ok(),
             "celo" => ssi::keccak_hash::hash_public_key(key).ok(),
@@ -546,7 +644,8 @@ impl DIDMethod for DIDPKH {
             "sol" => generate_sol(key),
             "btc" => generate_btc(key).ok(),
             "doge" => generate_doge(key).ok(),
-            _ => None,
+            // CAIP-10/CAIP-2 chain id
+            name => return generate_caip10_did(key, name).ok(),
         } {
             Some(addr) => addr,
             None => return None,
@@ -642,6 +741,17 @@ mod tests {
         );
     }
 
+    #[tokio::test]
+    async fn test_glyph_split() {
+        // Subslicing this expected Tezos address by byte range 0..3 would break a char boundary.
+        // https://doc.rust-lang.org/std/ops/struct.Range.html#impl-SliceIndex%3Cstr%3E
+        let bad_did = "did:pkh:tz:💣️";
+        let (res_meta, _doc_opt, _meta_opt) = DIDPKH
+            .resolve(bad_did, &ResolutionInputMetadata::default())
+            .await;
+        assert_ne!(res_meta.error, None);
+    }
+
     async fn test_resolve_error(did: &str, error_expected: &str) {
         let (res_meta, doc_opt, _meta_opt) = DIDPKH
             .resolve(did, &ResolutionInputMetadata::default())
@@ -652,51 +762,105 @@ mod tests {
 
     #[tokio::test]
     async fn resolve_did_pkh() {
+        // CAIP-10-based
         test_resolve(
-            "did:pkh:tz:tz1TzrmTBSuiVHV2VfMnGRMYvTEPCP42oSM8",
+            "did:pkh:tezos:NetXdQprcVkpaWU:tz1TzrmTBSuiVHV2VfMnGRMYvTEPCP42oSM8",
             include_str!("../tests/did-tz1.jsonld"),
         )
         .await;
         test_resolve(
-            "did:pkh:tz:tz2BFTyPeYRzxd5aiBchbXN3WCZhx7BqbMBq",
+            "did:pkh:tezos:NetXdQprcVkpaWU:tz2BFTyPeYRzxd5aiBchbXN3WCZhx7BqbMBq",
             include_str!("../tests/did-tz2.jsonld"),
         )
         .await;
         test_resolve(
-            "did:pkh:tz:tz3agP9LGe2cXmKQyYn6T68BHKjjktDbbSWX",
+            "did:pkh:tezos:NetXdQprcVkpaWU:tz3agP9LGe2cXmKQyYn6T68BHKjjktDbbSWX",
             include_str!("../tests/did-tz3.jsonld"),
         )
         .await;
         test_resolve(
-            "did:pkh:eth:0xb9c5714089478a327f09197987f16f9e5d936e8a",
+            "did:pkh:eip155:1:0xb9c5714089478a327f09197987f16f9e5d936e8a",
             include_str!("../tests/did-eth.jsonld"),
         )
         .await;
         test_resolve(
-            "did:pkh:celo:0xa0ae58da58dfa46fa55c3b86545e7065f90ff011",
+            "did:pkh:eip155:42220:0xa0ae58da58dfa46fa55c3b86545e7065f90ff011",
             include_str!("../tests/did-celo.jsonld"),
         )
         .await;
         test_resolve(
-            "did:pkh:poly:0x4e90e8a8191c1c23a24a598c3ab4fb47ce926ff5",
+            "did:pkh:eip155:137:0x4e90e8a8191c1c23a24a598c3ab4fb47ce926ff5",
             include_str!("../tests/did-poly.jsonld"),
         )
         .await;
         test_resolve(
-            "did:pkh:sol:CKg5d12Jhpej1JqtmxLJgaFqqeYjxgPqToJ4LBdvG9Ev",
+            "did:pkh:solana:4sGjMW1sUnHzSxGspuhpqLDx6wiyjNtZ:CKg5d12Jhpej1JqtmxLJgaFqqeYjxgPqToJ4LBdvG9Ev",
             include_str!("../tests/did-sol.jsonld"),
         )
         .await;
         test_resolve(
-            "did:pkh:btc:128Lkh3S7CkDTBZ8W7BbpsN3YYizJMp8p6",
+            "did:pkh:bip122:000000000019d6689c085ae165831e93:128Lkh3S7CkDTBZ8W7BbpsN3YYizJMp8p6",
             include_str!("../tests/did-btc.jsonld"),
         )
         .await;
         test_resolve(
-            "did:pkh:doge:DH5yaieqoZN36fDVciNyRueRGvGLR3mr7L",
+            "did:pkh:bip122:1a91e3dace36e2be3bf030a65679fe82:DH5yaieqoZN36fDVciNyRueRGvGLR3mr7L",
             include_str!("../tests/did-doge.jsonld"),
         )
         .await;
+        test_resolve(
+            "did:pkh:aleo:1:aleo1y90yg3yzs4g7q25f9nn8khuu00m8ysynxmcw8aca2d0phdx8dgpq4vw348",
+            include_str!("../tests/did-aleo.jsonld"),
+        )
+        .await;
+
+        // non-CAIP-10 (deprecated)
+        test_resolve(
+            "did:pkh:tz:tz1TzrmTBSuiVHV2VfMnGRMYvTEPCP42oSM8",
+            include_str!("../tests/did-tz1-legacy.jsonld"),
+        )
+        .await;
+        test_resolve(
+            "did:pkh:tz:tz2BFTyPeYRzxd5aiBchbXN3WCZhx7BqbMBq",
+            include_str!("../tests/did-tz2-legacy.jsonld"),
+        )
+        .await;
+        test_resolve(
+            "did:pkh:tz:tz3agP9LGe2cXmKQyYn6T68BHKjjktDbbSWX",
+            include_str!("../tests/did-tz3-legacy.jsonld"),
+        )
+        .await;
+        test_resolve(
+            "did:pkh:eth:0xb9c5714089478a327f09197987f16f9e5d936e8a",
+            include_str!("../tests/did-eth-legacy.jsonld"),
+        )
+        .await;
+        test_resolve(
+            "did:pkh:celo:0xa0ae58da58dfa46fa55c3b86545e7065f90ff011",
+            include_str!("../tests/did-celo-legacy.jsonld"),
+        )
+        .await;
+        test_resolve(
+            "did:pkh:poly:0x4e90e8a8191c1c23a24a598c3ab4fb47ce926ff5",
+            include_str!("../tests/did-poly-legacy.jsonld"),
+        )
+        .await;
+        test_resolve(
+            "did:pkh:sol:CKg5d12Jhpej1JqtmxLJgaFqqeYjxgPqToJ4LBdvG9Ev",
+            include_str!("../tests/did-sol-legacy.jsonld"),
+        )
+        .await;
+        test_resolve(
+            "did:pkh:btc:128Lkh3S7CkDTBZ8W7BbpsN3YYizJMp8p6",
+            include_str!("../tests/did-btc-legacy.jsonld"),
+        )
+        .await;
+        test_resolve(
+            "did:pkh:doge:DH5yaieqoZN36fDVciNyRueRGvGLR3mr7L",
+            include_str!("../tests/did-doge-legacy.jsonld"),
+        )
+        .await;
+
         test_resolve_error("did:pkh:tz:foo", ERROR_INVALID_DID).await;
         test_resolve_error("did:pkh:eth:bar", ERROR_INVALID_DID).await;
     }
@@ -991,8 +1155,8 @@ mod tests {
         credential_prove_verify_did_pkh(
             key_secp256k1_recovery.clone(),
             other_key_secp256k1.clone(),
-            "eth",
-            "#Recovery2020",
+            "eip155",
+            "#blockchainAccountId",
             &ssi::ldp::EcdsaSecp256k1RecoverySignature2020,
             None,
             None,
@@ -1003,17 +1167,29 @@ mod tests {
         credential_prove_verify_did_pkh(
             key_secp256k1_eip712sig.clone(),
             other_key_secp256k1.clone(),
-            "eth",
-            "#Recovery2020",
+            "eip155",
+            "#blockchainAccountId",
             &ssi::ldp::Eip712Signature2021,
             None,
             None,
         )
         .await;
 
+        // eth/epsig
+        credential_prove_verify_did_pkh(
+            key_secp256k1_eip712sig.clone(),
+            other_key_secp256k1.clone(),
+            "eip155",
+            "#blockchainAccountId",
+            &ssi::ldp::EthereumPersonalSignature2021,
+            None,
+            None,
+        )
+        .await;
+
         // eth/Eip712
         let eip712_domain: ssi::eip712::ProofInfo = serde_json::from_value(json!({
-          "messageSchema": {
+          "types": {
             "EIP712Domain": [
               { "name": "name", "type": "string" }
             ],
@@ -1043,7 +1219,7 @@ mod tests {
         }))
         .unwrap();
         let vp_eip712_domain: ssi::eip712::ProofInfo = serde_json::from_value(json!({
-          "messageSchema": {
+          "types": {
             "EIP712Domain": [
               { "name": "name", "type": "string" }
             ],
@@ -1077,7 +1253,7 @@ mod tests {
             "EIP712Info": [
               { "name": "domain", "type": "EIP712Domain" },
               { "name": "primaryType", "type": "string" },
-              { "name": "messageSchema", "type": "Types" },
+              { "name": "types", "type": "Types" },
             ],
             "Types": [
               { "name": "EIP712Domain", "type": "Type[]" },
@@ -1099,8 +1275,8 @@ mod tests {
         credential_prove_verify_did_pkh(
             key_secp256k1_eip712sig.clone(),
             other_key_secp256k1.clone(),
-            "eth",
-            "#Recovery2020",
+            "eip155",
+            "#blockchainAccountId",
             &ssi::ldp::EthereumEip712Signature2021,
             Some(eip712_domain),
             Some(vp_eip712_domain),
@@ -1111,8 +1287,8 @@ mod tests {
         credential_prove_verify_did_pkh(
             key_secp256k1_epsig.clone(),
             other_key_secp256k1.clone(),
-            "eth",
-            "#Recovery2020",
+            "eip155",
+            "#blockchainAccountId",
             &ssi::ldp::Eip712Signature2021,
             None,
             None,
@@ -1272,6 +1448,7 @@ mod tests {
 
     #[tokio::test]
     async fn verify_vc() {
+        // TODO: update these to use CAIP-10 did:pkh issuers
         test_verify_vc(include_str!("../tests/vc-tz1.jsonld"), 0).await;
         test_verify_vc(include_str!("../tests/vc-tz1-jcs.jsonld"), 1).await;
         test_verify_vc(include_str!("../tests/vc-eth-eip712sig.jsonld"), 0).await;
